/*************************************************************************/
/*                                                                       */
/* Program: dragon.c                                                     */
/*                                                                       */
/* Dragon curve program                                                  */
/* ====================                                                  */
/*                                                                       */
/* Algorithm by:   Henning von Rosen and Stefan Spaennare                */
/* Program by:     Stefan Spaennare, Lund, Sweden                        */
/* E-mail:         stefan@spaennare.se                                   */
/* First version:  October 1992                                          */
/* Latest update:  2005-07-23                                            */
/*                                                                       */
/*************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdio.h>
#include <graphics.h>
#include <conio.h>

double pi=3.14159265358979323;

void main()
{
   int gdriver = DETECT, gmode, errorcode;

   long x72,y72,x7,y7,i,n,k,v,v0,dd,nn1,nn2,q,nn,p;

   double v1,v2,c,r,dt,x,y,z,x1,y1,x0,y0,ss;

   clrscr();
   printf("Dragon curve\n\n");
   printf("\n");
   printf("Input power of 2 (15): ");
   scanf("%D",&p);
   printf("Input delay (1-100000): ");
   scanf("%D",&dd);
   clrscr();

   initgraph(&gdriver,&gmode, "");

   errorcode = graphresult();

   setcolor(1);

   line(0,0,639,0);
   line(639,0,639,479);
   line(639,479,0,479);
   line(0,479,0,0);

   setcolor(4);

   r=1.0;

   if (p > 15) {
      r=0.5;
   } /* if */

   c=2.0*r;
   x0=400.0;
   y0=370.0;
   v0=1;
   v=0;
   x=0.0;
   y=0.0;

   x72=(int)(x0+0.5);
   y72=(int)(y0+0.5);

   nn=1;
   for (i=1; i<=p; i++) {
     nn=2*nn;
   } /* for i */

   for (n=1; n<nn; n++) {

     k=1;

     do {
	nn1=(n-k) % (4*k);
	nn2=(n+k) % (4*k);
	k=2*k;
     } while ((nn1 != 0) && (nn2 != 0));

     if (nn2 == 0) {
	q=1;
     } /* if */

     if (nn1 == 0) {
	q=0;
     } /* if */

     if (v0 == 0) {
	if (v==1) {
	   if (q==0) {
	      x=x+c; v0=3; v=2;
	   }
	   else {
	      v0=1; v=2;
	   }
	}
	else {
	   if (q==0) {
	      v0=3; v=2;
	   }
	   else {
	      x=x-c; v0=1; v=2;
	   }
	}
     } /* if */

else if (v0 == 1) {
	if (v==0) {
	   if (q==0) {
	      v0=0; v=3;
	   }
	   else {
	      y=y-c; v0=2; v=3;
	   }
	}
	else {
	    if (q==0) {
	       y=y+c; v0=0; v=3;
	    }
	    else {
	       v0=2; v=3;
	    }
	}
     } /* if */

else if (v0 == 2) {
	if (v==1) {
	   if (q==0) {
	      v0=1; v=0;
	   }
	   else {
	      x=x+c; v0=3; v=0;
	   }
	}
	else {
	   if (q==0) {
	      x=x-c; v0=1; v=0;
	   }
	   else {
	      v0=3; v=0;
	   }
	}
     } /* if */

else if (v0 == 3) {
	if (v==0) {
	   if (q==0) {
	      y=y-c; v0=2; v=1;
	   }
	   else {
	      v0=0; v=1;
	   }
	}
	else {
	   if (q==0) {
	      v0=2; v=1;
	   }
	   else {
	      y=y+c; v0=0; v=1;
	   }
	}
     }  /* if */


     x7=(int)(x+x0+0.5);
     y7=(int)(y+y0+0.5);

     line(x7,y7,x72,y72);

     x72=x7;
     y72=y7;

     for (i=1; i<=dd; i++) ;

   } /* for n */

   getch();

   closegraph();

} /* End  */
