/***********************************************************************/
/*                                                                     */
/* Program: koch.c                                                     */
/*                                                                     */
/* The algorithm is taken from the book:                               */
/*                                                                     */
/* "Fractals for the Classroom" part one, Springer Verlag by           */
/* H-O. Peitgen, H. Juurgens and D. Saupe.                             */
/*                                                                     */
/*                                                                     */
/* Program by: Stefan Spaennare, Lund, June 1995                       */
/* Email: stefans@astro.lu.se                                          */
/* Latest update: 2004-06-23                                           */
/*                                                                     */
/***********************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <graphics.h>


double pi=3.14159265358979323;



void recu(xl,xr,yl,yr,r,lev,col)
double *xl,*xr,*yl,*yr;
double r;
long *lev;
long col;
{
   long xli,yli,xri,yri;

   double inv3;

   inv3=1.0/3.0;

   if (*lev == 1) {

      xli=(long)(xl[1]+0.5)+80;
      yli=(long)(yl[1]+0.5);
      xri=(long)(xr[1]+0.5)+80;
      yri=(long)(yr[1]+0.5);

      setcolor(col);
      line(xli,yli,xri,yri);

      return;

   }
   else {
      (*lev)--;

      xl[*lev]=xl[*lev+1];
      yl[*lev]=yl[*lev+1];
      xr[*lev]=inv3*xr[*lev+1]+2.0*inv3*xl[*lev+1];
      yr[*lev]=inv3*yr[*lev+1]+2.0*inv3*yl[*lev+1];

      recu(xl,xr,yl,yr,r,lev,col);

      xl[*lev]=xr[*lev];
      yl[*lev]=yr[*lev];
      xr[*lev]=0.5*xr[*lev+1]+0.5*xl[*lev+1]-r*(yl[*lev+1]-yr[*lev+1]);
      yr[*lev]=0.5*yr[*lev+1]+0.5*yl[*lev+1]+r*(xl[*lev+1]-xr[*lev+1]);

      recu(xl,xr,yl,yr,r,lev,col);

      xl[*lev]=xr[*lev];
      yl[*lev]=yr[*lev];
      xr[*lev]=2.0*inv3*xr[*lev+1]+inv3*xl[*lev+1];
      yr[*lev]=2.0*inv3*yr[*lev+1]+inv3*yl[*lev+1];

      recu(xl,xr,yl,yr,r,lev,col);

      xl[*lev]=xr[*lev];
      yl[*lev]=yr[*lev];
      xr[*lev]=xr[*lev+1];
      yr[*lev]=yr[*lev+1];

      recu(xl,xr,yl,yr,r,lev,col);

      (*lev)++;

      return;

   } /* if */

} /* recu */




int main(argc,argv)
int argc;
char *argv[];
{
   int gdriver = DETECT,gmode;

   double r,ss,yy;

   long lev,i,j,xsize,ysize,lev0,col,bgcol;

   double *xl,*xr,*yl,*yr;

clrscr();
printf("Input curve colour (0-15) (0): ");
scanf("%D",&col);
printf("Input background colour (0-15) (15): ");
scanf("%D",&bgcol);
printf("Input recursion level (5): ");
scanf("%D",&lev0);
clrscr();

initgraph(&gdriver,&gmode, "");


   xl=(double *)calloc(lev0+1,sizeof(double));
   xr=(double *)calloc(lev0+1,sizeof(double));
   yl=(double *)calloc(lev0+1,sizeof(double));
   yr=(double *)calloc(lev0+1,sizeof(double));


xsize=640;
ysize=480;

setcolor(bgcol);

for (i=0; i<xsize; i++) {
   line(i,0,i,ysize);
} /* for i */


ss=(double)(ysize)/1024.0;
r=0.29;
lev=lev0;

yy=750.0-sqrt(800.0*800.0-400.0*400.0);

xl[lev]=500.0*ss;
yl[lev]=yy*ss;
xr[lev]=900.0*ss;
yr[lev]=750.0*ss;

recu(xl,xr,yl,yr,r,&lev,col);

r=0.29;
lev=lev0;

xl[lev]=900.0*ss;
yl[lev]=750.0*ss;
xr[lev]=100.0*ss;
yr[lev]=750.0*ss;

recu(xl,xr,yl,yr,r,&lev,col);

r=0.29;
lev=lev0;

xl[lev]=100.0*ss;
yl[lev]=750.0*ss;
xr[lev]=500.0*ss;
yr[lev]=yy*ss;

recu(xl,xr,yl,yr,r,&lev,col);

getch();

closegraph();

} /* End */
